define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_EditorComponent",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dojo/text!./Templates/ChargeEditor.html",
    "thornberry/editor/property/PatientSearch",
    "thornberry/serverRequest",
    "thornberry/app/utils",
    "thornberry/form/searchBox",
    "dijit/form/ValidationTextBox",
    "dojo/store/Memory",
    "dojo/when",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dijit/form/Form",
    "dijit/form/Select",
    "dijit/form/DateTextBox",
    "dijit/form/CurrencyTextBox"
],function(
    declare,
    lang,
    arrayUtils,
    _EditorComponent,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _TableFieldEditorMixin,
    template,
    PatientSearch,
    serverRequest,
    utils,
    searchBox,
    ValidationTextBox,
    Memory,
    when,
    domStyle,
    domConstruct
){
    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin],{
        templateString: template,

        _requestInProgress: false,

        medId: "",

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.patientSearch = new PatientSearch({
                name: "ndocNum",
                reset: function() {
                    this.deserialize("");
                },
                buildRendering: function() {
                    var t = this;
                    t.inherited("buildRendering",arguments);
                    t.reqBkgdNode = t.searchInput.domNode;
                },
                _add: function(selection,noEvent) {
                    var t = this;
                    t.inherited("_add",arguments,[selection,true]);
                    if (selection !== null && !noEvent) {
                        t.emit("change",selection.value);
                    }
                }
            },t.patientNode);
            t.patientSearch.startup();

            t.medSearch = new searchBox({
                name: "medicationSearch",
                maxLength: 150,
                allowFreeText: true,
                getSearchResults: function(text) {
                    return serverRequest("Billing.Charge.Medication","SearchMeds",{searchText:text}).then(function(results) {
                        return new Memory({data:results});
                    });
                },
                _getValueAttr: function() {
                    return this.searchTextBox.get("value");
                },
                executeSearch: function() {
                    /* change from inherited: no escapeText() call on options */
                    var t = this;
                    if (ValidationTextBox.prototype.validator.call(t.searchTextBox,t.searchTextBox.get("value"),t.searchTextBox.constraints)) {
                        var searchString = t.searchTextBox.get("value");
                        if (searchString !== t._lastSearchString) {
                            t._lastSearchString = searchString;
                            
                            t.resultsSelect.domNode.length = 0;
                            t.resultsSelect.set("value","");
                            t.selectButton.set("disabled",true);
                            
                            if (searchString !== "") {
                                when(t.getSearchResults(searchString),function(store) {
                                    t.store = store;
                                    arrayUtils.forEach(store.data,function(item) {
                                        domConstruct.create("option",{label:(item.label || item.name),value:item.id},t.resultsSelect.domNode);
                                    });
                                });
                            }
                        }
                    }
                },
                onChange: function(value) {
                    if (lang.isArray(value)) {
                        value = value[0];
                    }
                    t.medId = (this.store.get(value) ? value : "");
                }
            },t.medSearchNode);
            t.medSearch.startup();

            t.addLogic("ndocNum",function(name,value) {
                t.setFormValue("medication","",true);
                t._requestInProgress = true;
                serverRequest("Billing.Charge.Medication","GetPatientMeds",value).then(function(results) {
                    t._requestInProgress = false;
                    t.setTable("patientMedications",results,["medication"]);
                    if (t.props.mode === "edit") {
                        t.getField("medication").set("value",t.props.value.medId);
                    }
                    t.invalidate("fieldState");
                },function(error) {
                    utils.showMessage(error);
                    t._requestInProgress = false;
                });
            },true);
            t.addLogic("medication",function(name,value) {
                domStyle.set(t.medSearchContainer,"display",(value === "[OTHER]" ? "" : "none"));
                if (value !== "[OTHER]") {
                    t.setFormValue("medicationSearch","");
                }
            },true);

            t.addRequirement(["ndocNum","medication","chargeDate","revenueCode","quantity","gross","unitOfMeasure"],true)
            t.addRequirement(["medicationSearch"],function(name,values) {
                return values["medication"] === "[OTHER]";
            });

            t.addDisabled("chargeType",true);
            t.addDisabled("medication",function(name,value,values) {return t._requestInProgress || !values["ndocNum"];});
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);

            domStyle.set(t.patientContainer,"display",(t.props.mode === "edit" ? "none" : ""));
        },

        _getValueAttr: function() {
            var t = this;
            var value = t.inherited(arguments);
            if (value.medication === "[OTHER]") {
                value.medId = t.medId;
                value.medName = value.medicationSearch;
            } else {
                value.medId = value.medication;
                value.medName = t.getField("medication").get("displayedValue");
            }
            /*if (t.props.mode === "edit" && value.medId === "") {
                value.medId = t.props.value.medId;
            }*/
            delete value.medicationSearch;
            delete value.medication;
            return value;
        }
    });
});